<?php
/*  
 * Analysis Console for Incident Databases (ACID)
 *
 * Author: Roman Danyliw <rdd@cert.org>, <roman@danyliw.com>
 *
 * Copyright (C) 2000 Carnegie Mellon University
 * (see the file 'acid_main.php' for license details)
 *
 * Purpose:   
 *
 */

/* AddSelectedToAG()
 *
 * Purpose: adds alerts to an alert group
 *
 * Arguments: $ag_add      => array of alerts to add
 *            $ag_add_def  => data in $ag_add is 1=(sid,cid), 2=signature, 3=sensor id
 *            $num_rows    => number of rows in the query 
 *            $ag_add_type => whether $ag_add_key parameter is a name or ID
 *            $ag_add_key  => key (i.e. name or ID ) of AG into which to add alerts
 *            $ag_method   => how to get the specific alerts to add
 *                            'Selected' / 'On Screen' alerts (i.e. use $ag_add
 *                            to get the specific alerts) 
 *                            'Entire Query' (use the SQL string to extract the cid/sid)
 *            $sql         => SQL string used to get the cid/sid when an entire query
 *                            must be added 
 *            $db          => DB object
 *            $limit_start => start record out of a record set to add alerts
 *            $limit_offset=> offset in a record set (from start) to add alerts
 */
function AddSelectedToAG($ag_add, $ag_add_def, $num_rows, $ag_add_type, $ag_add_key, $ag_method, $sql, $db,
                         $limit_start=-1, $limit_offset=-1)
{
  GLOBAL $debug_mode;

  if ( $debug_mode > 0 )
  {
     echo "<BR>==== ADD Alerts to AG========<BR>
           num_rows = $num_rows<BR>
           sql = $sql<BR>
           ag_add_type = $ag_add_type<BR>
           ag_add_key = $ag_add_key<BR>
           ag_method = $ag_method<BR>
           ag_add_def = $ag_add_def<BR>
           limit_start = $limit_start<BR>
           limit_offset = $limit_offset<BR>";
  }

  $ag_id = "";
  $added_alert_num = 0;
  $dup_cnt = 0;

  /* if the AG key is a name qualify it to an id */
  if ( $ag_add_type == "ag_by_name" )
     $ag_id = GetAGIDbyName($ag_add_key, $db);
  else if ( $ag_add_type == "ag_by_id" )
  {
     $ag_id = $ag_add_key;

     if ( VerifyAGID($ag_id, $db ) == 0 )
     {
        ErrorMessage("Uknown AG ID specified (AG probably does not exist)");
        $ag_id = "";
     }
  }
  else
     ErrorMessage("Unrecognized AG key type");

  if ( $ag_id != "" )
  {
     /* When pure alerts are being added to an AG, identifying
      * which specific alerts to use is easy.  However, when an
      * entire signature or sensor must be added to an AG, we must 
      * first lookup which alerts belong to the signature/sensor.
      */

     /* if adding only specific alerts */
     if ( $ag_add_def == 1 )  
        $num_alert_blobs = 1;
     /* else adding all alerts from a signature */
     else
        $num_alert_blobs = $num_rows;
 
     if ( $debug_mode > 0 ) echo "<BR>Adding Elements from ".sizeof($ag_add)." alert blobs<BR>";

     for ( $j = 0; $j < $num_alert_blobs; $j++ )
     {
        if ( ($ag_method == "Entire Query") || ($ag_add_def > 1) )
        {
           /* adding all alert for a signature */
           if ( $ag_add_def == 2 )
           {
              $sql = "SELECT sid, cid FROM event WHERE signature='".
                     rawurldecode($ag_add[$j])."'";
              $sql2 = "SELECT count(sid) FROM event WHERE signature='".
                      rawurldecode($ag_add[$j])."'";           
           }
           /* adding all alert for a sensor */
           else if ( $ag_add_def == 3 )
           {
              if ( $ag_add[$j] == "" ) $tmp = -1;  else $tmp = $ag_add[$j]; 
              $sql = "SELECT sid, cid FROM event WHERE sid=".$tmp;     /*ag_add[$j];*/  
              $sql2 = "SELECT count(sid) FROM event WHERE sid=".$tmp;  /*$ag_add[$j];*/
           }

           /* if adding alerts by signature or sensor, count the 
            * the number of alerts 
            */
           if ( $ag_add_def > 1 )
           {
              $result2 = $db->acidExecute($sql2);
              $myrow2 = $result2->acidFetchRow();
              $num_rows = $myrow2[0];
              $result2->acidFreeRows();
           }

           if ( $debug_mode > 0 )
              echo "$j = [using SQL $num_rows for blob ".$ag_add[$j]."]: $sql<BR>";

           if ( $limit_start == -1 )
              $result = $db->acidExecute($sql, -1, -1, false);
           else
              $result = $db->acidExecute($sql, $limit_start, $limit_offset, false);

           if ( $db->acidErrorMessage() != "" )
           {
              ErrorMessage("Error retrieving alert list to add the entire query to an AG");
              if ( $debug_mode > 0 )  ErrorMessage($db->acidErrorMessage());
              return -1;
           }           
        }

        for ( $i = 0; $i < $num_rows; $i++ )
        {
            if ( ($ag_add[$i] != "") || ($ag_method == "Entire Query") || ($ag_add_def > 1) )
            {
               if ( ($ag_method == "Entire Query") || ($ag_add_def > 1) )
               {
                  $myrow = $result->acidFetchRow();
                  $sid = $myrow[0];
                  $cid = $myrow[1];  
               }
               else 
                  GetQueryResultID($ag_add[$i], $seq, $sid, $cid);

               if ( $sid != "" )
               {
                  if ( $debug_mode > 0 ) echo $sid.' - '.$cid.'<BR>';
                  $sql2 = "INSERT INTO acid_ag_alert (ag_id, ag_sid, ag_cid) ".
                          "VALUES ('".$ag_id."','".$sid."','".$cid."');";
                  $db->acidExecute($sql2, -1, -1, false);

                  if ( $db->acidErrorMessage() != "" )
                     ++$dup_cnt; 
                  else
                     ++$added_alert_num;
               } 
           }
        }
        if ( ($method == "Entire Query") || ($ag_add_def > 1) )
           $result->acidFreeRows();
     }

     if ( $added_alert_num > 0 )
        ErrorMessage("Successfully added ".$added_alert_num." alert(s) to AG #".$ag_id);
     if ( $dup_cnt > 0 )
        ErrorMessage("Ignored ".$dup_cnt." duplicate alert(s) attempted to be added to AG #".$ag_id);
     if ( $added_alert_num == 0 && $dup_cnt == 0 )
        ErrorMessage("No alerts were selected");
  }
}

function GetAGIDbyName($ag_name, $db)
{
  $ag_id = "";
  $sql = "SELECT ag_id FROM acid_ag WHERE ag_name='".$ag_name."'";

  $result = $db->acidExecute($sql, -1, -1, false);

  if ( $db->acidErrorMessage() != "" )
     ErrorMessage("The specified AG name search is invalid.  Try again!");
  else if ( $result->acidRecordCount() < 1 )
     ErrorMessage("The specified AG does not exist.");
  else
  {
        $myrow = $result->acidFetchRow();
        $ag_id = $myrow[0]; 
        $result->acidFreeRows();
  }

  return $ag_id;
}

function GetAGNamebyID($ag_id, $db)
{
  $ag_name = "";
  $sql = "SELECT ag_name FROM acid_ag WHERE ag_id='".$ag_id."'";

  $result = $db->acidExecute($sql, -1, -1, false);

  if ( $db->acidErrorMessage() != "" )
     ErrorMessage("The specified AG ID search is invalid.  Try again!");
  else if ( $result->acidRecordCount() < 1 )
     ErrorMessage("The specified AG does not exist.");
  else
  {
        $myrow = $result->acidFetchRow();
        $ag_id = $myrow[0]; 
        $result->acidFreeRows();
  }

  return $ag_name;
}

function VerifyAGID($ag_id, $db)
{
  $sql = "SELECT ag_id FROM acid_ag WHERE ag_id='".$ag_id."'";
  $result = $db->acidExecute($sql, -1, -1, false);

  if ( $db->acidErrorMessage() != "" )
  {
     ErrorMessage("Error looking up an AG ID");
     return 0;
  }
  else if ( $result->acidRecordCount() < 1 )
     return 0;
  else
  {
     $result->acidFreeRows();
     return 1;
  }
}

function ClearSelectedAlerts($ag_id, $alert_del, $num_rows, &$del_method, $sql, &$num_result_rows, $db,
                              $limit_start=-1, $limit_offset=-1)
{
  GLOBAL $debug_mode;

  $del_alert_num = 0;
  $del_table_list[0] = "acid_ag_alert";

  if ( $debug_mode > 0 )
  {
     echo "==== CLEAR Alerts ========<BR>";
     echo "num_rows = $num_rows<BR>
           ag_id = $ag_id<BR>
           del_method = $del_method<BR>
           sql = $sql<BR>
           limit_start = $limit_start<BR>";
  }

  if ( $del_method == "Entire Query" )
  {
     if ( $limit_start == -1 )
        $result = $db->acidExecute($sql, -1, -1, false);
     else
        $result = $db->acidExecute($sql, $limit_start, $limit_offset, false);

     if ( $db->acidErrorMessage() != "" )
     {
        ErrorMessage("Error retrieving alert list to clear");
        return -1;
     }
  }

  if ( $limit_start != -1 )    $num_rows = $limit_offset;

  for ( $i = 0; $i < $num_rows; $i++ )
  {
      if ( $alert_del[$i] != "" || $del_method == "Entire Query")
      {
         if ( $del_method == "Entire Query" )
         {
            $myrow = $result->acidFetchRow();
            $sid = $myrow[0];
            $cid = $myrow[1];  
         }
         else 
            GetQueryResultID($alert_del[$i], $seq, $sid, $cid);

         echo $sid.' - '.$cid.'<BR>';
         for ( $j = 0; $j < 1; $j++ )
         {
            $sql2 = "DELETE FROM ".$del_table_list[$j].
                    " WHERE ag_sid=".$sid." AND ag_cid=".$cid." AND ag_id=".$ag_id;
            $db->acidExecute($sql2);

            if ( $db->acidErrorMessage() != "" )
               ErrorMessage("Error Deleting Alert"." ".$del_table_list[$j]);
            else if ( $j == 0 ) 
               ++$del_alert_num; 
         } 
     }
  }

  if ( $method == "Entire Query" )
     $result->acidFreeRows();

  if ( $del_alert_num > 0 )
  {
     ErrorMessage("Successfully cleared ".$del_alert_num." alert(s)");
     /* Reset the alert count that the query is re-executed to reflect the deletion */
     $num_result_rows -= $del_alert_num;
     $del_method = "Query DB";
  }

  if ( $del_alert_num == 0 )
     ErrorMessage("No alerts were selected or the Clear was not successful"); 

  if ( $debug_mode > 0 )
     echo "==== CLEAR Alerts END ========<BR>";
}

?>


